@@ -7,6 +7,7 @@ from contextlib import nullcontext
 from pathlib import Path
 from typing import Union, BinaryIO, TYPE_CHECKING, Type, Optional, Generator
 
+
 from ....helper import (
     get_compress_ctx,
     decompress_bytes,
@@ -19,6 +20,20 @@ if TYPE_CHECKING:
     from .... import Document, DocumentArray
 
 
+class LazyRequestReader:
+    def __init__(self, r):
+        self._data = r.iter_content(chunk_size=1024 * 1024)
+        self.content = b''
+
+    def __getitem__(self, item: slice):
+        while len(self.content) < item.stop:
+            try:
+                self.content += next(self._data)
+            except StopIteration:
+                return self.content[item.start : -1 : item.step]
+        return self.content[item]
+
+
 class BinaryIOMixin:
     """Save/load an array to a binary file."""
 
@@ -51,7 +66,7 @@ class BinaryIOMixin:
             and `compress=lz4`.
         """
 
-        if isinstance(file, io.BufferedReader):
+        if isinstance(file, (io.BufferedReader, LazyRequestReader)):
             file_ctx = nullcontext(file)
         elif isinstance(file, bytes):
             file_ctx = nullcontext(file)
@@ -93,12 +108,7 @@ class BinaryIOMixin:
 
         from .... import Document
 
-        if _show_progress:
-            from rich.progress import track as _track
-
-            track = lambda x: _track(x, description='Deserializing')
-        else:
-            track = lambda x: x
+        from rich.progress import track
 
         with file_ctx as f:
             version_numdocs_lendoc0 = f.read(9)
@@ -107,7 +117,9 @@ class BinaryIOMixin:
             # 8 bytes (uint64)
             num_docs = int.from_bytes(version_numdocs_lendoc0[1:9], 'big', signed=False)
 
-            for _ in track(range(num_docs)):
+            for _ in track(
+                range(num_docs), description='Deserializing', disable=not _show_progress
+            ):
                 # 4 bytes (uint32)
                 len_current_doc_in_bytes = int.from_bytes(
                     f.read(4), 'big', signed=False
@@ -155,18 +167,16 @@ class BinaryIOMixin:
             version = int.from_bytes(d[0:1], 'big', signed=False)
             # 8 bytes (uint64)
             num_docs = int.from_bytes(d[1:9], 'big', signed=False)
-            if show_progress:
-                from rich.progress import track as _track
 
-                track = lambda x: _track(x, description='Deserializing')
-            else:
-                track = lambda x: x
+            from rich.progress import track
 
             # this 9 is version + num_docs bytes used
             start_pos = 9
             docs = []
 
-            for _ in track(range(num_docs)):
+            for _ in track(
+                range(num_docs), description='Deserializing', disable=not show_progress
+            ):
                 # 4 bytes (uint32)
                 len_current_doc_in_bytes = int.from_bytes(
                     d[start_pos : start_pos + 4], 'big', signed=False
@@ -261,6 +271,7 @@ class BinaryIOMixin:
         if protocol == 'protobuf-array' or protocol == 'pickle-array':
             compress_ctx = get_compress_ctx(compress, mode='wb')
         else:
+            # delegate the compression to per-doc compression
             compress_ctx = None
 
         with (_file_ctx or io.BytesIO()) as bf:
@@ -279,35 +290,19 @@ class BinaryIOMixin:
                 elif protocol == 'pickle-array':
                     f.write(pickle.dumps(self))
                 elif protocol in ('pickle', 'protobuf'):
-                    # Binary format for streaming case
-                    if _show_progress:
-                        from rich.progress import track as _track
+                    f.write(self._stream_header)
 
-                        track = lambda x: _track(x, description='Serializing')
-                    else:
-                        track = lambda x: x
+                    from rich.progress import track
 
-                    # V1 DocArray streaming serialization format
-                    # | 1 byte | 8 bytes | 4 bytes | variable | 4 bytes | variable ...
-
-                    # 1 byte (uint8)
-                    version_byte = b'\x01'
-                    # 8 bytes (uint64)
-                    num_docs_as_bytes = len(self).to_bytes(8, 'big', signed=False)
-                    f.write(version_byte + num_docs_as_bytes)
-
-                    for d in track(self):
-                        # 4 bytes (uint32)
-                        doc_as_bytes = d.to_bytes(protocol=protocol, compress=compress)
-
-                        # variable size bytes
-                        len_doc_as_bytes = len(doc_as_bytes).to_bytes(
-                            4, 'big', signed=False
+                    for d in track(
+                        self, description='Serializing', disable=not _show_progress
+                    ):
+                        f.write(
+                            d._to_stream_bytes(protocol=protocol, compress=compress)
                         )
-                        f.write(len_doc_as_bytes + doc_as_bytes)
                 else:
                     raise ValueError(
-                        f'protocol={protocol} is not supported. Can be only `protobuf`,`pickle`,`protobuf-array`,`pickle-array`.'
+                        f'protocol={protocol} is not supported. Can be only `protobuf`, `pickle`, `protobuf-array`, `pickle-array`.'
                     )
 
             if not _file_ctx:
@@ -362,3 +357,16 @@ class BinaryIOMixin:
         _show_progress: bool = False,
     ) -> str:
         return base64.b64encode(self.to_bytes(protocol, compress)).decode('utf-8')
+
+    @property
+    def _stream_header(self) -> bytes:
+        # Binary format for streaming case
+
+        # V1 DocArray streaming serialization format
+        # | 1 byte | 8 bytes | 4 bytes | variable | 4 bytes | variable ...
+
+        # 1 byte (uint8)
+        version_byte = b'\x01'
+        # 8 bytes (uint64)
+        num_docs_as_bytes = len(self).to_bytes(8, 'big', signed=False)
+        return version_byte + num_docs_as_bytes
