@@ -23,9 +23,7 @@ from ....helper import dataclass_from_dict
 from ..registry import _REGISTRY
 
 if TYPE_CHECKING:
-    from ....types import (
-        DocumentArraySourceType,
-    )
+    from ....types import DocumentArraySourceType, ArrayType
 
 
 @dataclass
@@ -277,10 +275,30 @@ class BackendMixin(BaseBackendMixin):
         :param value: document to create a payload for
         :return: the payload dictionary
         """
-        if value.embedding is not None:
+        return dict(
+            data_object={'_serialized': value.to_base64(**self._serialize_config)},
+            class_name=self._class_name,
+            uuid=self._map_id(value.id),
+            vector=self._map_embedding(value.embedding),
+        )
+
+    def _map_id(self, doc_id: str):
+        """the function maps doc id to weaviate id
+
+        :param doc_id: id of the document
+        :return: weaviate object id
+        """
+        # appending class name to doc id to handle the case:
+        # daw1 = DocumentArrayWeaviate([Document(id=str(i), text='hi') for i in range(3)])
+        # daw2 = DocumentArrayWeaviate([Document(id=str(i), text='bye') for i in range(3)])
+        # daw2[0, 'text'] == 'hi' # this will be False if we don't append class name
+        return str(uuid.uuid5(uuid.NAMESPACE_URL, doc_id + self._class_name))
+
+    def _map_embedding(self, embedding: 'ArrayType'):
+        if embedding is not None:
             from ....math.ndarray import to_numpy_array
 
-            embedding = to_numpy_array(value.embedding)
+            embedding = to_numpy_array(embedding)
 
             if embedding.ndim > 1:
                 embedding = np.asarray(embedding).squeeze()
@@ -292,25 +310,7 @@ class BackendMixin(BaseBackendMixin):
                 embedding = [embedding[0]]
         else:
             embedding = None
-
-        return dict(
-            data_object={'_serialized': value.to_base64(**self._serialize_config)},
-            class_name=self._class_name,
-            uuid=self._wmap(value.id),
-            vector=embedding,
-        )
-
-    def _wmap(self, doc_id: str):
-        """the function maps doc id to weaviate id
-
-        :param doc_id: id of the document
-        :return: weaviate object id
-        """
-        # appending class name to doc id to handle the case:
-        # daw1 = DocumentArrayWeaviate([Document(id=str(i), text='hi') for i in range(3)])
-        # daw2 = DocumentArrayWeaviate([Document(id=str(i), text='bye') for i in range(3)])
-        # daw2[0, 'text'] == 'hi' # this will be False if we don't append class name
-        return str(uuid.uuid5(uuid.NAMESPACE_URL, doc_id + self._class_name))
+        return embedding
 
     def _get_storage_infos(self) -> Dict:
         return {
